function [rgb] = hs1_to_rgb(h, s)
    %hs1_to_rgb: array version of HSV to RGB converter with V=1
    % ref: https://github.com/python/cpython/blob/master/Lib/colorsys.py
    
    r = ones(size(h));
    g = ones(size(h));
    b = ones(size(h));
    % compute some "magical" values
    i = uint8(h * 6.0);
    f = (h * 6.0) - double(i);
    p = 1.0 - s;
    q = 1.0 - s .* f;
    t = 1.0 - s .* (1.0 - f);
    i = mod(i, 6);
    % i = 0: v=1, t, p
    g(i == 0) = t(i == 0);
    b(i == 0) = p(i == 0);
    % i = 1: q, v=1, p
    r(i == 1) = q(i == 1);
    b(i == 1) = p(i == 1);
    % i = 2: p, v=1, t
    r(i == 2) = p(i == 2);
    b(i == 2) = t(i == 2);
    % i = 3: p, q, v=1
    r(i == 3) = p(i == 3);
    g(i == 3) = q(i == 3);
    % i = 4: t, p, v=1
    r(i == 4) = t(i == 4);
    g(i == 4) = p(i == 4);
    % i = 5: v=1, p, q
    g(i == 5) = p(i == 5);
    b(i == 5) = q(i == 5);

    rgb = cat(3, r, g, b);
end

