clear all;
img_dir = './';  % point to image folder
% image examples are from http://vision.middlebury.edu/flow/data/
imgname0 = strcat(img_dir, 'frame10.png'); 
imgname1 = strcat(img_dir, 'frame11.png');
levels = [4, 2, 1];  % downscale factors for each scale
nb_warps = 5;  % number of linearized warping steps
nbiter_pdhg = 100;  % number of pdhg iterations to perform
lamb = 7.0;  % lambda coefficient
visualize_stepwise_energies = false;
visualize_stepwise_updates = false;

% load and display normalized images
img0 = im2double(imread(imgname0));
img1 = im2double(imread(imgname1));
[height, width] = size(img0);
img0 = (img0-min(img0(:))) ./ (max(img0(:)) - min(img0(:)));
img1 = (img1-min(img1(:))) ./ (max(img1(:)) - min(img1(:)));
figure(1);
imshowpair(img0, img1);
title('before optical flow');

% initialize flow in x, y directions
ux = zeros(2, 2);
uy = zeros(2, 2);

% mutli-scale iterate coarse-to-fine
for level = levels
    fprintf('downscale factor %.2f\n', level);
    % resize images to current level
    h = height / level;
    w = width / level;
    i0 = imresize(imgaussfilt(img0, level), [h, w], 'bicubic');
    i1 = imresize(imgaussfilt(img1, level), [h, w], 'bicubic');
    %% upscale previous flow
    [ux, uy] = flowresize(ux, uy, [h, w]);
    % perform multiple linearized warping
    for i = 1:nb_warps
        fprintf('- warp step %d\n', i);
        % get warped image
        i1w = imwarp(i1, cat(3, ux, uy));
        % if required, draw status before update
        if visualize_stepwise_updates
            figure(2);
            title('before/after optim');
            subplot(1, 3, 1);
            imshowpair(i0, i1w);
        end
        % update flow using PDHG
        [ux, uy, energies] = update_flow(i0, i1w, ux, uy, lamb, nbiter_pdhg);
        % if required, draw status after update and the optical flow
        if visualize_stepwise_updates
            subplot(1, 3, 2);
            imshowpair(i0, imwarp(i1, cat(3, ux, uy)));
            subplot(1, 3, 3);
            flowshow_color(ux, uy);
        end
        % if required, plot evolution of primal energy
        if visualize_stepwise_energies
            figure(3);
            title('primal energy');
            plot(1:size(energies, 1), energies);
        end
    end
end

figure(4);
title('before/after optical flow');
subplot(131);
imshowpair(img0, img1);
subplot(132);
img1w = imwarp(img1, cat(3, ux, uy));
imshowpair(img0, img1w);
subplot(133);
flowshow_color(ux, uy);
